import re
import time
import logging
import Game
import requests
from bs4 import BeautifulSoup
import pandas
import os

logger = logging.getLogger("my_logger")
logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s', level=logging.DEBUG)

class XgpReccomender:

    def __init__(self):
        """Create database of all avaiable Xbox Game Pass games"""
        self.my_taste = self.taste_meter(rate_weight=1)

        with open('xbox_database', 'w') as f:
            if os.stat('xbox_database').st_size == 0:
                f.write('Name:Developer:Genre:Metascore:MyScore\n')

            rated_games = self.rated_games_list()
            for game in self.get_xgp_games():
                if game.name not in rated_games:
                    f.write(f"{re.sub('-', ' ', game.name)}:{game.developer}:{', '.join(game.genres)}:{game.metascore}:{game.my_score}\n")
        
        self.database = self.create_db()

    def get_xgp_games(self):
        """Getting available Xbox Game Pass games"""
        logger.info("Getting available Xbox Game Pass games...")
        page_number = 1
        gamenames_set = set()

        while True:
            true_achievements_url = f"https://www.trueachievements.com/xbox-game-pass/games?page={page_number}"
            page = requests.get(true_achievements_url)
            soup = BeautifulSoup(page.content, 'html.parser')
            site_elems = soup.findAll('tr', id=re.compile('^tr'))
            if not site_elems:
                break
            for site_elem in site_elems:
                for a in site_elem.find_all('a', href=True):
                    game_name = a['href'].split('/')[2] # eg. game_name = ['', 'game', 'Life-Is-Strange-2', 'achievements]
                    gamenames_set.add(game_name)
            page_number += 1
        
        games_set = set()
        logger.info(f'Number of all available games: {len(gamenames_set)}...')
        for count, game in enumerate(gamenames_set):
            logger.info(f'Looking for info about game number: {count}...')
            games_set.add(Game.Game(name=game, my_taste=self.my_taste))
        return games_set

    @staticmethod
    def rated_games_list():
        """Create rated games list"""
        logger.info("Creating rated games list...")
        rated_games = []
        with open('rated_games', 'r') as f:
            for line in f:
                game_name = line.rstrip().split(':')[0]
                rated_games.append(game_name)
        return rated_games

    @staticmethod
    def taste_meter(rate_weight=1):
        """Computing my game taste"""
        logger.info("Computing my game taste...")
        taste_meter = {}
        with open('rated_games', 'r') as f:
            for line in f:
                game_name, rating = line.rstrip().split(':')
                game = Game.Game(name=game_name, my_taste=None)
                for genre in game.genres:
                    if genre not in taste_meter:
                        taste_meter[genre] = rate_weight * int(rating)
                    else:
                        taste_meter[genre] += rate_weight * int(rating)

                if game.developer not in taste_meter:
                    taste_meter[game.developer] = rate_weight * int(rating)
                else:
                    taste_meter[game.developer] += rate_weight * int(rating)
        return taste_meter

    @staticmethod
    def create_db():
        pandas.set_option('display.width', 1000)
        pandas.set_option('colheader_justify', 'center')
        pandas.set_option('display.max_rows', None)

        database = pandas.read_csv('xbox_database', delimiter=':')
        database = database.sort_values('MyScore', ascending=False)
        database = database[database['Metascore'] != 0]

        return database


    def generate_html(self):
        html_string = '''
        <html>
          <head><title>HTML Pandas Dataframe with CSS</title></head>
          <link rel="stylesheet" type="text/css" href="df_style.css"/>
          <body>
            {table}
          </body>
        </html>.
        '''

        # OUTPUT AN HTML FILE
        with open('game_ranking.html', 'w') as f:
            f.write(html_string.format(table=self.database.to_html(classes='mystyle')))



import re
import time
import logging
import requests
from bs4 import BeautifulSoup


logger = logging.getLogger("my_logger")
logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s', level=logging.DEBUG)


class Game:
    """Game class containing basic info about certain game: name, metascore, developer, genre"""

    def __init__(self, name, my_taste):
        self.name = name
        self.metascore, self.developer, self.genres = self.get_game_info()
        self.my_taste = my_taste
    
    def __str__(self):
        return f'Name: {self.name}, metascore: {self.metascore}, developer: {self.developer}, genre: {self.genre}'

    def __repr__(self):
        return f'{self.name}: {self.metascore}'

    # Checking metascore of a certain game
    def get_game_info(self):
        """Get info about the game: MetaScore, Developer, Genre."""
        metacritic_url = f"https://www.metacritic.com/game/xbox-one/{self.name.lower()}"
        headers = {
            'User-Agent': "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.113 Safari/537.36"}
        page = requests.get(metacritic_url, headers=headers)
        if page.status_code == 404:
            metacritic_url = f"https://www.metacritic.com/game/xbox-360/{self.name.lower()}"
            page = requests.get(metacritic_url, headers=headers)
        elif page.status_code == 404:
            time.sleep(10)
            page = requests.get(metacritic_url, headers=headers)

        soup = BeautifulSoup(page.content, 'html.parser')
        site_elems = soup.find(class_=re.compile('^metascore_w xlarge game'))

        if not site_elems:
            return 0, '-', '-'

        metascore = int(site_elems.find('span').text)

        site_elems = soup.find(class_='summary_detail developer')
        developer = site_elems.find(class_='data').text
        developer = developer.strip()

        site_elems = soup.find(class_='summary_detail product_genre')
        genres_site_elems = site_elems.findAll('span', class_='data')
        genres = set()
        for genre in genres_site_elems:
            genres.add(genre.get_text())
        return metascore, developer, genres

    # Compute game score given by my game taste
    @property    
    def my_score(self):
        """Method compute MyScore"""
        my_score = self.metascore
        # Check which of genres a game is and which a developer made the game
        for key, value in self.my_taste.items():
            if key in self.genres or key in self.developer:
                my_score += value
        return my_score



from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name='xgp-reccomender',
    version='1.0.2',
    description='App searches for avaiable Xbox Game Pass games and helps to choose what to play next.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/aowczarek618/xgp-reccomender',
    packages=find_packages(where='.'),
    python_requires='>=3.5, <4',
    install_requires=['requests', 'beautifulsoup4', 'pandas'],
    entry_points={
        "console_scripts": [
            "xgp-reccomender=xgp_reccomender.main:main",
        ]
    },
    project_urls={
        'Bug Reports': 'https://github.com/aowczarek618/xgp-reccomender/issues',
        'Source': 'https://github.com/aowczarek618/xgp-reccomender',
    },
)